import 'dart:convert';
import 'dart:io';
import 'dart:async';

import 'dart:typed_data';

void handleMsg(msg) async {
  if (msg is Uint8List) {
    try {
      final files = await Directory.current.list().toList();
      final newFileName = "app-${files.length}.png";
      final newFile = File(newFileName);
      await newFile.writeAsBytes(msg);
      print('Received screenshot: $newFileName');
    } catch (e) {
      print('Received screenshot, but failed to save to disk : $e');
    }
  }
}

Future main() async {
  runZoned(() async {
    final websockets = <WebSocket>[];

    stdin
        .transform(utf8.decoder)
        .transform(const LineSplitter())
        .listen((line) {
      final splits = line
          .split(" ")
          .map((x) => x.trim())
          .where((x) => x.length > 0)
          .toList();

      if (splits.isNotEmpty) {
        try {
          final message = _createMessage(splits);
          final json = jsonEncode(message);
          websockets.forEach((ws) {
            print("Sending command to device [$ws] : $message");
            ws.add(json);
          });
        } catch (e) {
          print("Bad command: $e");
        }
      }
    });

    var server = await HttpServer.bind('192.168.1.11', 4041);
    print("[RemoteDevicePreview] Serving at ${server.address}:${server.port}");
    await for (var request in server) {
      var socket = await WebSocketTransformer.upgrade(request);
      print("Device connected [$socket]");
      websockets.add(socket);
      socket.listen(handleMsg, onDone: () => websockets.remove(socket));
    }
  }, onError: (e) => print("An error occurred."));
}

Map<String, dynamic> _createMessage(List<String> commandArguments) {
  final command = commandArguments[0];
  Map<String, dynamic> arguments = {};

  if (command == "setDevice") {
    arguments["name"] = commandArguments[1];
  }

  return {
    "command": command,
    "arguments": arguments,
  };
}
